/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>  // for stat()
#include <sys/stat.h>   // for stat()
#include <unistd.h>     // for stat()

#include "./fileindexutil.h"   // for class CRC32.
#include "./FileIndexReader.h"

extern "C" {
  #include "libhw1/Assert333.h"
}

namespace hw3 {

FileIndexReader::FileIndexReader(std::string filename,
                                 bool validate) {
  // Stash a copy of the index file's name.
  filename_ = filename;

  // Open a (FILE *) associated with filename.  Crash on error.
  file_ = fopen(filename_.c_str(), "rb");
  Assert333(file_ != NULL);

  // Make the (FILE *) be unbuffered.  ("man setbuf")
  // MISSING:


  // Read the magic number header field from the index file, verify
  // that it is correct.  Crash if not.
  uint32_t magic_number;
  // MISSING:


  // Read the checksum header field from the index file.
  uint32_t checksum;
  // MISSING:


  // Read the doctable size field from the index file, and store
  // it in the doctable_size_ member variable of this object instance.
  // (We're not going to remind you to convert between network and
  // host order any more; you'll have to remember to do it yourself
  // in the right locations from now on.)
  // MISSING:

  // Read the index size field from the index file and store it in the
  // index_size_ member variable.
  // MISSING:

  // Make sure the index file's length lines up with the header fields.
  struct stat f_stat;
  Assert333(stat(filename_.c_str(), &f_stat) == 0);
  Assert333((uint32_t) f_stat.st_size ==
            (uint32_t) (FILEINDEX_HEADER_SIZE +
                        doctable_size_ +
                        index_size_));

  if (validate) {
    // Re-calculate the checksum, make sure it matches that in the header.
    // Use fread() to read the checksum and pass the bytes you read into
    // the crcobj.  Note you don't need to do any host/network order
    // conversion, since we're doing this byte-by-byte.
    CRC32 crcobj;
    uint8_t buf[512];
    uint32_t left_to_read = doctable_size_ + index_size_;
    while (left_to_read > 0) {
      // MISSING:
    }
    Assert333(crcobj.GetFinalCRC() == checksum);
  }

  // Everthing looks good; we're done!
}

FileIndexReader::~FileIndexReader() {
  // Close the (FILE *).
  Assert333(fclose(file_) == 0);
}

DocTableReader FileIndexReader::GetDocTableReader() {
  // The docid->name mapping starts at offset FILEINDEX_HEADER_SIZE in
  // the index file.  Be sure to dup the (FILE *) rather than sharing
  // it across objects, just so that we don't end up with the possibility
  // of threads contending for the (FILE *) and associated with race
  // conditions.
  return DocTableReader(FileDup(file_), FILEINDEX_HEADER_SIZE);
}

IndexTableReader FileIndexReader::GetIndexTableReader() {
  // The index (word-->docid table) mapping starts at offset
  // (FILEINDEX_HEADER_SIZE + doctable_size_) in the index file.  Be
  // sure to dup the (FILE *) rather than sharing it across objects,
  // just so that we don't end up with the possibility of threads
  // contending for the (FILE *) and associated race conditions.
  return IndexTableReader(FileDup(file_),
                          FILEINDEX_HEADER_SIZE + doctable_size_);
}

}  // namespace hw3
